/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _ATTRIBUTEH
#define _ATTRIBUTEH
class Attribute
{
public:
  Attribute(const string & name, 
            int type_id,
            bool qualified = false,
            string defaultVal = "", 
            string fixedVal = "", 
            bool use = true)
    :attName(name),
     dval(defaultVal),
     fval(fixedVal)
  {

    //initialize the attribute's properties
    attType = type_id;
    bQualified = qualified;
    attUse = use;
  }
  Attribute(void)
  {
    attType = 0;
    bQualified = false;
    attUse = false;
  }
  string getName() const
  {
    return attName;
  }
  int getType() const
  {
    return attType;
  }
  bool isRequired() const
  {
    return attUse;
  }
  string & defaultVal()
  {
    return dval;
  }
  string & fixedVal()
  {
    return fval;
  }
  bool isQualified()const 
  {
    return bQualified;
  }
  Attribute & operator = (const Attribute & a)
  {
    attName = a.attName;
    attType = a.attType;
    bQualified = a.isQualified();
    dval = a.dval;
    fval = a.fval;
    attUse = a.attUse;
    return *this;
  }
private:string attName;
  int attType;
  bool bQualified, attUse;
  string dval, fval;
};
#ifdef LOGGING
ostream & operator << (ostream & stream, Attribute & a);
#endif                                            /*  */
#endif                                            /*  */
