/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _ELEMENTH
#define _ELEMENTH

#define UNBOUNDED 1000
class Element
{
public:
  Element(const string & name, 
          int type_id, 
          int min = 1, 
          int max = 1, 
          bool qualified = false, 
          string def = "", 
          string fixed ="")
    :elemName(name),
     dval(def),
     fval(fixed)
  {
    
    //initialize the Element's properties
    elemType = type_id;
    bQualified = qualified;
    minOccurs = min;
    maxOccurs = max;
  }

  Element(void)
  {

    //initialize the Element's properties
    elemType = 0;
    bQualified = false;
    minOccurs = 1;
    maxOccurs = 1;
    nOccurrences = 0;
  }
  void setType(int id)
  {
    elemType = id;
  }
  void setGroupId(int group)
  {
    groupId = group;
  }
  int getGroupId() const 
  {
    return groupId;
  }
  string getName() const
  {
    return elemName;
  }
  int getType() const
  {
    return elemType;
  }
  int max() const 
  {
    return maxOccurs;
  }
  int min() const
  {
    return minOccurs;
  }
  string & defaultVal()
  {
    return dval;
  }
  string & fixedVal()
  {
    return fval;
  }
  bool isQualified() const
  {
    return bQualified;
  }
  Element& operator = (const Element & e)
  {
    elemName = e.elemName;
    elemType = e.elemType;
    bQualified = e.isQualified();
    dval = e.dval;
    fval = e.fval;
    minOccurs = e.min();
    maxOccurs = e.max();
    nOccurrences = e.nOccurrences;
    groupId = e.groupId;
    return *this;
  }
  void min(int m)
  {
    minOccurs=m;
  }
  void max(int m)
  {
    maxOccurs=m;
  }

  int nOccurrences;
private:string elemName;
  int elemType;
  bool bQualified;
  string dval, fval;
  int minOccurs, maxOccurs;

  //the group ID indicates which composition group it belongs to.
  int groupId;
};
#ifdef LOGGING
ostream & operator << (ostream & stream, Element & e);
#endif
#endif                                            /*  */
