/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _SCHEMAVALIDATORH
#define _SCHEMAVALIDATORH

#include <schemaparser/SchemaParser.h>
#include <schemaparser/SchemaParserException.h>
#include <schemaparser/TypeContainer.h>

//class for validating a schema instance
class SchemaValidator
{
public:
  /** @name Constructors and Destructors */
  //@{
  /**
   * The constructor for SchemaValidator
   * @param pointer to the schema parser instance for the schema
   */
  SchemaValidator(const SchemaParser * sp);
  ~SchemaValidator();

  //@}

  /** @name methods for validating schema instance*/
  //@{
  /**
   * validate .This is the main function to validate any type with its instance
   * @param  XmlPullParser instance of the xml stream containing the
   instance of the schema type
   * @param pointer to the type whose instance must be validated
   * @param type container to be used (can be null)
   * @return type container containing an instance of the schema type
   */
  TypeContainer *validate(XmlPullParser * xpp, int typeId,
                          TypeContainer * ipTc = 0);
  TypeContainer *validate(string & val , int typeId,
                          TypeContainer * ipTc = 0); //fro simple types

  //@}
private:
  /**
   * validateSimpleContent .validates simple  schema types
   */
  TypeContainer * validateSimpleContent(string & val, int basetype,
                                        TypeContainer * ipTc);
  void extractSimpleType(string & val, int basetype,
                         TypeContainer * ipTc, SimpleType * st);
  bool  findElement(list < Element > & pElemList, list < Element >::iterator &pElem, 
                    string name, bool rewind = true);
  void error(string);
  const SchemaParser *sParser;
};
#endif
