/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _SIMPLETYPEH
#define _SIMPLETYPEH
#include <list>

#include <schemaparser/XSDType.h>
#include <xmlpull/Qname.h>
#include  <utils/utils.h>
#include <schemaparser/SchemaParserException.h>


//union to store the values for various facets
typedef union
{
    int length;
    struct
    {
        int minlen, maxlen;
    } lenRange;
    int numEnums;
    int wsp;
    struct
    {
        int maxinc, mininc, maxex, minex;
    } valRange;
    int tot;
    int frac;
    const char *pattern;
} facetValueType;

class SimpleType:public XSDType
{
    public:
    /** @name Constructors and Destructors */
    //@{
    /**
     * @param the namespace uri of the simple type
     */
    SimpleType(string);
    ~SimpleType();
    //@}
    
    /** @name Various Getter methods*/
    //@{

    /**
     * getName 
     * @return NCName of the simple type 
     */

    string getName() const;
    /**
     * getQName 
     * @return QName (namespace qualified name)of the simple type 
     */
    Qname getQname() const;
    /**
     * getContentModel 
     * @return content model which is always SIMPLE 
     */
    int getContentModel() const;
    /**
     * getBaseType 
     * @return :if this is a derived type ,this method returns the 
     *          base type from which this is derived
     */
    int getBaseType()const ;
    /**
     * getTypeId
     * @return :type identifier
     */
    int getTypeId() const;
    /**
     * isAnonymous
     * @return :is this an anoymous type
     */
    bool isAnonymous() const;

   //@}

    //@{ Various setter and utility methods

    void setName(string s);
    void setContentModel(int cm);
    void setBaseType(int id);
    void setTypeId(int id);
    void setAnonymous(bool flag);
    bool isvalidFacet(string facet);
    void setFacetValue(string val);
    bool isValidInt(int val);
    bool isValidFloat(float val);
    bool isValidString(string val);
    bool getFacetValue(int facet, void **val);

    //@}
    
//enum for facet Ids
    enum
    {
        NONE = 0, LENGTH = 0x1, MINLEN = 0x2, MAXLEN = 0x4, ENUM =
        0x8, WSP = 0x10, MAXINC = 0x20, MININC = 0x40, MAXEX =
        0x80, MINEX = 0x100, TOT = 0x200, FRAC = 0x400, PATTERN = 0x800
    };

//whitespace values
    enum
    {
        PRESERVE, REPLACE, COLLAPSE
    };
#ifdef LOGGING
    void print(ostream & out);
#endif
    private:
    string name, nsUri;
    int basetype;
    int typeId;                                   //Unique Type id
    int contentModel;
    bool restriction;
    int facetId[5];
    int numFacets;
    bool anonymous;
    list < string > enum_list;
    int *validFacets;
    facetValueType facetValue;
    void error(string msg);
};
#endif                                            /*  */
