/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//This class is a recursive storage container for any xml data type
//It stores the actual values occuring in an XML Schema Instance ,
//for a given type defined in the Schema.
#ifndef _TYPECONTAINERH
#define  _TYPECONTAINERH
#include <map>

#include <utils/utils.h>
#include <schemaparser/SchemaParser.h>

class TypeContainer;

/*
 The maximum occurences of a child within  a parent that can be
  supported is 10
 */
typedef struct
{
    TypeContainer *tc[10];
    int count;

//Count maintains the last accessed container
    int num;

//num is the number of occurrences of this child
} Containers;
class TypeContainer
{
public:TypeContainer(int typeId, const SchemaParser * sp);
  ~TypeContainer();
  TypeContainer *getAttributeContainer(string elemName, bool create =
                                       false);
  TypeContainer *getChildContainer(string elemName, bool create = false);

  //If this is a container for atomic type this method 
  //returns a void* to the type .The actual type  must be found using getTypeId()

  void *getValue();
   
  //Various set methods
  void setValue(string & sValue)
  {
    Value.sValue = new string(sValue);
  }
  void setValue(int iValue)
  {
    Value.iValue = new int (iValue);
  }
  void setValue(char cValue)
  {
    Value.cValue = new char (cValue);
  }

  //void setValue(unsigned int uiValue);
  void setValue(long lValue)
  {
    Value.lValue = new long (lValue);
  }
  void setValue(unsigned long ulValue)
  {
    Value.ulValue = new unsigned long (ulValue);
  }

  //void setValue(short shValue);
  //void setValue(unsigned short usValue);
  void setValue(float fValue)
  {
    Value.fValue = new float;
    *(Value.fValue) = fValue;
  }
  void setValue(double dbValue)
  {
    Value.dbValue = new double;
    *(Value.dbValue) = dbValue;
  }
  void setValue(bool bValue)
  {
    Value.bValue = new bool;
    *(Value.bValue) = bValue;
  }

  //       void setValue(Date dValue);
  //   void setValue(     DateTime dtValue);
  //       void setValue(Time tValue);
  void setValue(Qname & qnValue)
  {
    Value.qnValue = new Qname(qnValue);
  }

  //   void setValue(Uri uriValue );
  int getTypeId()
  {
    return typeId;
  }
  bool isSimple()
  {
    return (sParser->getType(typeId))->isSimple();
  }
protected:
  //The unique id of the type for which this holds data
  int typeId;
  map < string, Containers * >childList;
  map < string, TypeContainer * >attributeList;
  const SchemaParser *sParser;
  union
  {
    string *sValue;
    int *iValue;
    unsigned int *uiValue;
    long *lValue;
    unsigned long *ulValue;
    short *shValue;
    unsigned short *usValue;
    float *fValue;
    double *dbValue;
    bool *bValue;
    char *cValue;

    //              Date *dValue;
    //              DateTime *dtValue;
    //              Time *tValue;
    Qname *qnValue;

    //      Uri *uriValue ;
  } Value;
private:
  void deleteValue();
  TypeContainer * tcTable[20];
  int nTc;
};
#endif                                            /*  */
