/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _TYPESTABLEH
#define _TYPESTABLEH

#include <string>
#include <vector>
#include <map>
#include <sstream>
#include <utils/utils.h>
#include <schemaparser/SimpleType.h>
#include <schemaparser/ComplexType.h>

//List of all XSD types
class TypesTable
{
public:

  TypesTable();
  ~TypesTable();

  //Realease all the memory
  void clean();

  //add a new type and returns its id
  int addType(XSDType * type);
  int addExtType(XSDType * type, int id);

  //get the type id of a type
  int getTypeId(Qname & name, bool create = false);

  //store an external reference to another schema
  int addExternalTypeId(Qname & type, XSDType * pType);
  int getNumExtRefs()
  {
    return nExtRefs;
  }
  Qname & getExtRefName(int index)
  {
    return m_extRefs[index].qname;
  }
  int getExtRefType(int index)
  {
    return m_extRefs[index].localTypeId;
  }
  void resolveForwardElementRefs(const string & name, Element & e);
  void resolveForwardAttributeRefs(const string & name, Attribute & a);

  //get the path to the descendant element
  int getCompleteXpath(int elemId, string & childName, int *xPath,
                       int limits, int &offset);

  //returns the pointer to a type whose id is given
  XSDType *getTypePtr(int id) const
  {

    // this is a basic XSD type
    if (id < XSDType::ANYURI + 1 || id > XSDType::ANYURI + numTypes)
      return 0;
    return typesArray[id - (SimpleType::ANYURI + 1)];
  }
  int getNumTypes(void) const
  {
    return numTypes;
  } bool detectUndefinedTypes(void);
  void setTargetNamespace(string Uri)
  {
    m_tnsUri = Uri;
  }
  string getTargetNamespace(void)
  {
    return m_tnsUri;
  }
  void printUndefinedTypes(ostream & out);

#ifdef LOGGING
  //for logging purposes
  void printTypes(ostream & out);
#endif 
private:
  //contains a pointer to the actual type
  XSDType ** typesArray;

  //maintains a map of all user defined type names and their ids
  map < string, int >Id;

  //This is a map of all types defined in XML Schema
  map < string, int >basicTypes;
  int currentId;
  int numTypes, nSize;
  string m_tnsUri;
  struct extRefs
  {
    int localTypeId;
    Qname qname;
  } m_extRefs[5];
  int nExtRefs;
  void ensureCapacity();
};
#endif                                            /*  */
