/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifndef _MESSAGEH
#define  _MESSAGEH

#include <string>
#include <xmlpull/Qname.h>
#include <wsdlparser/WsdlException.h>
#include <wsdlparser/wsdl.h>

//Implementation of Wsdl message element

typedef struct
{
  string pname;
  refType reftype;
  int type_id;
  int schema_id;
  int pExtId;
} Part;
class Message
{
public:

  Message(void)
  {
    nParts = 0;
    doc=0;
    nElems = nAtts = 0;
  }
  ~Message()
  {
  };

  /** @name Various Getter methods*/
  //@{

  /**
   * getName
   * @return name of the Message
   */
  string getName() const
  {
    return name;
  }
  /**
   * getNumParts
   * @return number of parts in the message
   */
  int getNumParts(void) const                   
  {
    return nParts;
  }
  /**
   * getPartIndex
   * @param the name of the part
   * @return index of the of the part whose name is passed
   *         index:0..nParts-1
   */
  int getPartIndex(string & nam) const         
  {
    for (int i = 0; i < nParts; i++)
      {
        if (parts[i].pname == nam)
          return i;
      }
    return -1;
  }

  /* getPartName
   * @param the index  of the part       index:0..nParts-1
   * @return name of the of the part whose index is passed
   */
  string getPartName(int index) const
  {
    return parts[index].pname;
  }
  /* getPartRefType
   * @param the index  of the part       index:0..nParts-1
   * @return ref type of the part ..is it element="Qname" or type="Qname"
   */
  refType getPartRefType(int index) const
  {
    return parts[index].reftype;
  }
  refType getPartRefType(string & nam) const
  {
    for (int i = 0; i < nParts; i++)
      {
        if (parts[i].pname == nam)
          return parts[i].reftype;
      }
    return Type;
  }

  /* getPartType
   * @param the index  of the part       index:0..nParts-1
   * @return type id  of the part
   *         for ex if we have <part name="one" type="xsd:int">
   *         the id returned represents xsd:int,the schema type for integers
   */
  int getPartType(int index) const
  {
    return parts[index].type_id;
  }
  int getPartType(string & nam) const
  {
    for (int i = 0; i < nParts; i++)
      {
        if (parts[i].pname == nam)
          return parts[i].type_id;
      }
    return 0;
  }

  /* getPartType
   * @param the index  of the part       index:0..nParts-1
   * @return schema id of the schema parser instance to which the type id  of the part   belongs
   *         for ex if we have <part name="one" type="xs2:mytype"> t
   *         the id returned represents the schema parser instance which parsed schema with namespace "xs2"
   */
  int getPartContentSchemaId(int index) const
  {
    return parts[index].schema_id;
  }
  int getPartContentSchemaId(string & nam) const
  {
    for (int i = 0; i < nParts; i++)
      {
        if (parts[i].pname == nam)
          return parts[i].schema_id;
      }
    return -1;
  }
  /**
   * getDocumentation
   *   
   */
  const  string* getDocumentation() const
  {
    return (const string*)doc;
  }


  //@}

  /** @name Various Setter methods*/
  //@{
  void setName(string nam)
  {
    this->name = nam;
  } 
   
  void addPart(string pname, refType reftype, int type_id,
               int schema_id, int eId = 0)
  {
    if (type_id == 0 && reftype == Type)
      throw
        WsdlException
        ("Message::addPart() :wrong type or no type suplied");
    parts[nParts].pname = pname;
    parts[nParts].reftype = reftype;
    parts[nParts].type_id = type_id;
    parts[nParts].schema_id = schema_id;
    parts[nParts].pExtId = eId;
    nParts++;
  }
  void addExtElement(int ident)
  {
    if (nElems > MAX_EXT_ELEM)
      return;
    extElems[nElems++] = ident;
  }
  void addExtAtts(int ident)
  {
    if (nAtts > MAX_EXT_ATTS)
      return;
    extAtts[nAtts++] = ident;
  }
  void setDocumentation(string* s)
  {     
    doc=s;
  }


  //@}
  void print(ostream & out)
  {
    int i;
    out << id << dbsp << name << endl;
    out << nParts << endl;
    for (i = 0; i < nParts; i++)
      {
        out << parts[i].pname << dbsp;
        out << parts[i].reftype << dbsp << parts[i].type_id << dbsp;
        out << parts[i].pExtId << endl;
      }
    out << nElems << dbsp << dbsp;
    for (i = 0; i < nElems; i++)
      out << extElems[i] << dbsp;
    out << endl;
    out << nAtts << dbsp << dbsp;
    for (i = 0; i < nAtts; i++)
      out << extAtts[i] << dbsp;
    out << blk;
  }
private:
  string name;
  int nParts;
  int id;
  Part parts[MAX_PARTS];
  int extElems[MAX_EXT_ELEM], nElems;
  int extAtts[MAX_EXT_ATTS], nAtts;
  string * doc;
};
#endif                                            /*  */
