/* 
 * wsdlpull - A C++ parser  for Wsdl  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _OPERATIONH
#define  _OPERATIONH

#include <wsdlparser/wsdl.h>
#include <xmlpull/Qname.h>
#include <wsdlparser/WsdlException.h>
#include <wsdlparser/Message.h>

//class for Wsdl operation element
class Operation
{
public:

  Operation()
  {
    m_type = INVALID;
    nElems = nAtts = 0;
    m_inMessage = m_outMessage = m_faultMessage = 0;
    doc=0;       
  }
  ~Operation()
  {
  }
  /** @name Various Getter methods*/
  //@{
  /**
   * getName
   * @return name of the Operation
   */
  string getName() const
  {
    return m_name;
  }
  /**
   * getMessage
   * @param mesage type ,Input,Output or Fault
   * @return pointer to the message
   */
  const Message *getMessage(int type) const     
  {
    if (type == Input)
      return m_inMessage;

    else if (type == Output)
      return m_outMessage;

    else if (type == Fault)
      return m_faultMessage;

    else
      return 0;
  }
  /**
   * getOptype
   * @return type of the operation in,in-out,out,out-in
   */
  Optype getType() const                        
  {
    return m_type;
  }
  /**
   * getDocumentation
   *   
   */
  const  string* getDocumentation() const
  {
    return (const string*)doc;
  }

  //@}/** @name Various Setter methods*///@{
  void setMessage(const Message * message, messageType type)
  {
    if (message == 0)
      throw WsdlException("Invalid message name");
    if (type == Input)
      {
        m_inMessage = message;
        if (m_type == INVALID)
          m_type = IN;

        else if (m_type == OUT)
          m_type = OUT_IN;

        else
          m_type = INVALID;
      }
    else if (type == Output)
      {
        m_outMessage = message;
        if (m_type == INVALID)
          m_type = OUT;

        else if (m_type == IN)
          m_type = IN_OUT;

        else
          m_type = INVALID;
      } else if (type == Fault)
        m_faultMessage = message;
  }
  void setName(string name)
  {
    m_name = name;
  }
  void addExtElement(int id)
  {
    if (nElems > MAX_EXT_ELEM)
      return;
    extElems[nElems++] = id;
  }
  void addExtAtts(int id)
  {
    if (nAtts > MAX_EXT_ATTS)
      return;
    extAtts[nAtts++] = id;
  }
  void setDocumentation(string* s)
  {
    doc=s;
  }


  //@}
  void print(ostream & out)
  {
    int i;
    out << m_Id << dbsp << m_name << endl;
    out << m_type << endl;
    out << m_inMessage << dbsp << m_outMessage << dbsp <<
      m_faultMessage << endl;
    out << nElems << dbsp << dbsp;
    for (i = 0; i < nElems; i++)
      out << extElems[i] << dbsp;
    out << endl;
    out << nAtts << dbsp << dbsp;
    for (i = 0; i < nAtts; i++)
      out << extAtts[i] << dbsp;
    out << blk;
  }
private:
  string m_name;
  const Message *m_inMessage, *m_outMessage, *m_faultMessage;
  Optype m_type;
  int m_Id;
  int extElems[MAX_EXT_ELEM], nElems;
  int extAtts[MAX_EXT_ATTS], nAtts;
  string * doc;
};
#endif                                            /*  */
