/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _PORTTYPEH
#define _PORTTYPEH

#include <wsdlparser/wsdl.h>
#include <xmlpull/Qname.h>
#include <wsdlparser/Operation.h>

//This class implements a port type of WSDL .
class PortType
{
public:
  PortType()
  {
    nOps = nElems = nAtts = 0;
    doc=0;
  }

  ~PortType() { }
  /** @name Various Getter methods*/
  //@{
  /**
   * getName
   * @return name of the Operation
   */
  string getName() const
  {
    return m_name;
  }
  int getNumOps(void) const
  {
    return nOps;
  }
  /**
   * getOperation
   * @param index of the operation index:0..nOps-1
   * @return pointer to  the Operation
   */
  const Operation *getOperation(int index) const
  {
    return ops[index];
  }
  /**
   * getOperation
   * @param name of the operation
   * @return pointer to  the Operation
   */
  const Operation *getOperation(Qname & name) const
  {
    for (int i = 0; i < nOps; i++)
      {
        if (ops[i]->getName() == name.getLocalName())
          return ops[i];
      }
    return 0;
  }

  /**
   * getOperations
   * @return pointer to  list of Operations in the port type
   */
  list < const Operation *>*getOperations(Qname & portType)
  {
    list < const Operation *>*plOps = 0;
    if (nOps > 0)
      {
        plOps = new list < const Operation *>();
        for (int i = 0; i < nOps; i++)
          plOps->push_back(ops[i]);
      }
    return plOps;
  }
  /**
   * getDocumentation
   *   
   */
  const  string* getDocumentation() const
  {
    return (const string*)doc;
  }


  //@}

  /** @name Various Setter methods*/
  //@{
  void setName(string name)
  {
    m_name = name;
  }
  void addOp(const Operation * op)
  {
    ops[nOps++] = op;
  }
  void addExtElement(int id)
  {
    if (nElems > MAX_EXT_ELEM)
      return;
    extElems[nElems++] = id;
  }
  void addExtAtts(int id)
  {
    if (nAtts > MAX_EXT_ATTS)
      return;
    extAtts[nAtts++] = id;
  }
  void setDocumentation(string* s)
  {
    doc=s;
  }

  //@}
private:
  string m_name;
  int nOps;
  const Operation *ops[MAX_OPS_PER_PT];
  int extElems[MAX_EXT_ELEM], nElems;
  int extAtts[MAX_EXT_ATTS], nAtts;
  string * doc;
};
#endif                                            /*  */
