/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */


#ifndef _SOAPEXTH
#define _SOAPEXTH

#include <iostream>
#include <fstream>

#include <wsdlparser/WsdlExtension.h>
#include <wsdlparser/WsdlParser.h>
#include <schemaparser/SchemaValidator.h>

const string httpTransport = "http://schemas.xmlsoap.org/soap/http";
const string httpBinding = "http://schemas.xmlsoap.org/wsdl/http/";

class Soap:public WsdlExtension
{
public:

  Soap();
  ~Soap();

  /*
    Returns the namespace URI of the wsdl
    extensibility elements that it can handle.
  */
  string getNamespace()
  {
    return sNamespace;
  }
  void setNamespacePrefix(string pre)
  {
    sNsPrefix = pre;
  }
  string getNamespacePrefix()
  {
    return sNsPrefix;
  }
  bool isNamespaceHandler(string & ns)
  {
    return (ns == sNamespace);
  }
  XmlPullParser *getExtensibilitySchema(void);
  void setSchemaParser(SchemaParser * spe)
  {
    mySchemaParser = spe;
    mySchemaValidator = new SchemaValidator(mySchemaParser);
  }
  // parent is the Wsdl parent element type under which the extensibility element has come
  int handleElement(int parent, XmlPullParser *);

  //attName is the extensibility attribute
  int handleAttribute(int parent, string attName, XmlPullParser *);

  //returns a valid extensibilty element
  int getElementName(int id);
  int getElemAttribute(int id, int att_num);
  int getElemAttributeValue(int id, int att_num);

  //returns a valid extensibility attribute
  int getAttributeName(int id);

  //this is the start of all ids that must be used for elems/attributes in this namespace
  void setStartId(int id)
  {
    startId = id;
  }
  int getStartId()
  {
    return startId;
  }
  virtual void setWsdlParser(WsdlParser * wp)
  {
    wParser = wp;
  }
  bool wasUsed()
  {
    return (wParser != 0);
  }
  void serialize(ostream & out);
  void getSoapOperationInfo(int elemId, char **soapAction, int &style);
  void getSoapBodyInfo(int elemId, char **ns, int &use);
  void getSoapHeaderInfo(int elemId, int &schemaId, int &typeId);
  void getServiceLocation(int elemId, char **location);

  //TODO add more methods like this
  bool isSoapBody(int id);
  bool isSoapHeader(int id);

  /*
    Enums used in soap
  */
  typedef enum
    {
      LITERAL, ENC
    } Encoding;
  typedef enum
    {
      RPC, DOC
    } Style;
  typedef enum
    {
      NONE, HTTP, SMTP
    } Transport;
private:void error(string);
  int processBinding(TypeContainer * t);
  int processOp(int, TypeContainer * t);
  int processBody(int, TypeContainer * t);
  int processHeader(int, TypeContainer * t);
  int processFault(int, TypeContainer * t);
  int processAddress(int parent, TypeContainer * t);
  string sNamespace, sNsPrefix, sTitle;
  int startId;
  SchemaParser *mySchemaParser;
  SchemaValidator *mySchemaValidator;
  WsdlParser *wParser;
  struct
  {
    int typeId;
    int index;
  } idTable[MAX_EXT_XML];
  int idCounter;
  struct
  {
    int wsdlOpId;
    char *soapAction;
    int style;
  } m_ops[MAX_OPS];
  int nOps;
  struct
  {
    int messageId;
    Encoding use;
    int encodingStyle;
    char *urn;
  } m_body[MAX_OPS * MAX_PTYPES];
  int nMsgs;
  struct
  {
    int schema;
    int typeId;
  } m_header[MAX_OPS * MAX_PTYPES];
  int nHeader;
  Transport m_transport;
  Style m_style;
  char *m_location[MAX_PTYPES];
  int nLocation;
};
inline int Soap::getElementName(int id)
{
    if (id < startId || id > (startId + idCounter - 1))
        return 0;
    return idTable[id - startId].typeId;
}


inline int Soap::getAttributeName(int id)
{
    if (id < startId || id > (startId + idCounter - 1))
        return 0;
    return idTable[id - startId].typeId;
}
#endif                                            /*  */
