/*
 *  XmlPullParser API for C++ is free software!!
 *  Initial Author:  Vivek Krishna
 *
 */
#ifndef _XMLPULLPARSERH
#define _XMLPULLPARSERH

#include <map>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
using namespace std;

/** A simple, pull based XML parser.  */
const int RESIZE_BUFFER = 16;

//features
const string FEATURE_PROCESS_NAMESPACES =
"http://xmlpull.org/v1/doc/features.html#process-namespaces";
const string FEATURE_REPORT_NAMESPACE_ATTRIBUTES =
"http://xmlpull.org/v1/doc/features.html#report-namespace-prefixes";
const string FEATURE_PROCESS_DOCDECL =
"http://xmlpull.org/v1/doc/features.html#process-docdecl";
const string FEATURE_VALIDATION =
"http://xmlpull.org/v1/doc/features.html#validation";
const string NO_NAMESPACE = "";
class XmlPullParser
{
private:string unexpected_eof;
  string illegal_type;
  int LEGACY;
  int XML_DECL;

  // general
  string version;
  bool standalone;

  //   private bool reportNspAttr;
  bool processNsp;
  bool relaxed;
  map < string, string > entityMap;
  int depth;
  vector < string > elementStack;
  vector < string > nspStack;
  int *nspCounts;
  int nspSize;

  // source
  istream & reader;
  string encoding;
  char *srcBuf;
  int srcPos;
  int srcCount;
  int srcBuflength;

  //    private bool eof;
  int line;
  int column;

  // txtbuffer
  char *txtBuf;
  int txtPos;
  int txtBufSize;

  // Event-related
  int type;
  string text;
  bool isWspace;
  string Ns;
  string prefix;
  string name;
  bool degenerated;
  int attributeCount;
  vector < string > attributes;

  /**
   * A separate peek buffer seems simpler than managing
   * wrap around in the first level read buffer */
  int peek[2];
  int peekCount;
  bool wasCR;
  bool unresolved;
  bool token;
public:  XmlPullParser (istream & is);

  //constructor
  XmlPullParser (void);
  ~XmlPullParser (void);

  //enum for event types
  enum
    {
      START_DOCUMENT =
      0, END_DOCUMENT, START_TAG, END_TAG, TEXT, CDSECT, ENTITY_REF,
      IGNORABLE_WHITESPACE, PROCESSING_INSTRUCTION, COMMENT, DOCDECL
    };
private:void commonInit (void);
  void initBuf (void);

  //returns the state name as  a string
  string state (int eventType);
  bool isProp (string n1, bool prop, string n2);
  bool adjustNsp ();                            //throws XmlPullParserException
  string *ensureCapacity (string * arr, int required);
  void exception (string desc);                 //throws XmlPullParserException

  /**
   * common base for next and nextToken. Clears the state, except from
   * txtPos and whitespace. Does not set the type variable */
  void nextImpl ();                             //throws IOException, XmlPullParserException
  int parseLegacy (bool push);

  //throws IOException, XmlPullParserException

  /** precondition: &lt! consumed */
  void parseDoctype (bool push);

  /* precondition: &lt;/ consumed */
  void parseEndTag ();
  int peekType ();
  string get (int pos);
  void push (int c);

  /** Sets name and attributes */
  void parseStartTag (bool xmldecl);

  /** result: isWspace; if the setName parameter is set,
      the name of the entity is stored in "name" */

  //vivek
  void pushEntity ();

  /** types:
      '<': parse to any token (for nextToken ())
      '"': parse to quote
      ' ': parse to whitespace or '>'
  */
  void pushText (int delimiter, bool resolveEntities);
  void read (char c);
  int read ();

  /** Does never read more than needed */
  int peekbuf (int pos);
  string readName ();
  void skip ();
public:
  //--------------- public part starts here... ---------------
  bool getFeature (string feature);
  string getInputEncoding ();
  void defineEntityReplacementText (string entity, string value);
  int getNamespaceCount (int depth);
  string getNamespacePrefix (int pos);
  string getNamespaceUri (int pos);
  string getNamespace (string prefix);
  
  int getDepth ();
  string getPositionDescription ();
  int getLineNumber ()
  {
    return line;
  }
  int getColumnNumber ()
  {
    return column;
  }
  bool isWhitespace ();
  string getText ();
  const char *getTextCharacters (int *poslen);
  string getNamespace ()
  {
    return Ns;
  }
  string getName ()
  {
    return name;
  }
  string getPrefix ()
  {
    return prefix;
  }
  bool isEmptyElementTag ();
  int getAttributeCount ()
  {
    return attributeCount;
  }
  string getAttributeType (int index)
  {
    return "CDATA";
  }
  bool isAttributeDefault (int index)
  {
    return false;
  }
  string getAttributeNamespace (int index);
  string getAttributeName (int index);
  string getAttributePrefix (int index);
  string getAttributeValue (int index);
  string getAttributeValue (string ns, string name);
  int getEventType ()
  {
    return type;
  }
  int next ();
  int nextToken ();

  //----------------------------------------------------------------------
  // utility methods to make XML parsing easier ...
  int nextTag ();
  void require (int type, string ns, string name);
  string nextText ();
  void setFeature (string feature, bool value);

  /*	 string  TYPES[11] ;*/
};
#endif                                            /*  */
